#!/usr/bin/perl -w #################################################################
#
# Script Name:
#
# Description:
#
# Parameters:
#
# Invocation:
#
# Invokes:
#
# Return Codes:
#
# Dependencies:
#

#
### GLOBAL DECLARATIONS & DEFAULT ###
#

$working_dir = "/opt/Tivoli/SWRepository/PatchMgt";
$exclusion_list = $working_dir . "/Targets/Exceptions.txt";
$act_delimiter = "%";
$swdpack_check = 1;

my $LocalHostOid = `objcall 0.0.0 getattr HostLocation`;
if ( $? == 0 && $LocalHostOid && $LocalHostOid ne "" )
{
    $targ_mn = `idlcall $LocalHostOid SysAdmin::PolicyDrivenBase::_get_label`;
    $targ_mn =~ s/"//g;
}

if ( !$targ_mn || $targ_mn eq "" )
{
    print "\nUnable to get the local ManagedNode\n\n";
    exit 1;
}

my ($os_type, $os_comment) = split(" ", `uname -a`);
if  ( $os_type eq "Windows_NT" )
{
    $sysRoot = $ENV{"SystemRoot"};
    $InitFile = "$sysRoot/system32/drivers/etc/Tivoli/wcfgplng.ini";
    $nullDevice = "nul";
}
else
{
    $InitFile = "/etc/Tivoli/wcfgplng.ini";
    $nullDevice = "/dev/null";
}

my($pre); my(%xml_list);
$query_lib_name = "ACTIVITY_PLANNER_REPORTING";

if ( ! $ARGV[0] )
{
    print "\nPlease give a base name for this Distribution.\n";
    print "Please Keep it short as it will proceed the name of all of\n";
    print "your APM plans\n\n";
    print "Usage: $0 <Distribution name>\n\n";
    exit 2;
}
else
{
    $pre = $ARGV[0];
}

#
### INITIALIZATION ###
#

get_init("$InitFile");

check_and_create_query_library();

#
### MAIN ###
#

print "Are you using an exclusion list? (y/n)\n";
my $exception = <STDIN>;

if ( $exception =~ m/^y/i )
{
    if ( ! -e $exclusion_list )
    {
        print "\nExclusion file: $exclusion_list not found!\n\n";
        exit 3;
    } 
    else 
    {
        print "Exclusion file: $exclusion_list will be used\n";
    }
} 
else 
{
    print "OK, No exclusion list will be used\n";
}

print "Would you like to import the plans into APM after the XML is Created? (y/n)\n";
my $response = <STDIN>;

if ( -d $working_dir . "/" . $pre ) 
{
    `rm -r "$working_dir\/$pre"`;
}

## reset the umask so that it is group readable
umask(0002); 
mkdir($working_dir . "/" . $pre, 0775) 
    or die "Can't make $working_dir/$pre: $!\n";

my @mstcfg = get_config("$working_dir/cfg/Master_CFG.lst");

foreach (@mstcfg)
{
    my($xml,$id,$cfg_list,$exclusion_flag) = @{$_};

    $cfg_file_name = $working_dir . "/cfg/" . $cfg_list;
    @cfg_file = get_config($working_dir . "/cfg/" . $cfg_list);

    foreach (@cfg_file) 
    {
        undef my %act_hash;
        undef my $unique_package;

        my @cfg_line = @{$_};
        my $plan_xml_name = shift(@cfg_line);
        my $file = shift(@cfg_line);
        my $depot = shift(@cfg_line);
        foreach $act_pack (@cfg_line)
        {
            if ($act_pack && $act_pack ne "")
            {
                if ($act_pack eq $act_delimiter)
                {
                    print "\nInvalid activity-package definition '$act_pack' in the $cfg_file_name file.\n\n";
                    exit 4;
                }
                my ($t_act, $t_pack) = split($act_delimiter, $act_pack);
                if (! $t_pack || $t_pack eq "")
                {
                    $t_pack = $t_act;
                    undef $t_act;
                }
                if (!$t_act || $t_act eq "")
                {
                    if (!%act_hash && !$unique_package)
                    {
                        $unique_package = $t_pack;
                    }
                    else
                    {
                        my $cfg_line_str = join(":",@cfg_line);
                        print "\nInvalid activity-package definitions '$cfg_line_str'\n";
                        if (!%act_hash)
                        {
                            print "Unique SWD package set twice: '$unique_package' and '$t_pack'\n\n";
                        }
                        else
                        {
                            print "Unique SWD package '$t_pack' set togheter with single acitivities\n\n";
                        }
                        exit 5;
                    }
                }
                else
                {
                    if ($unique_package)
                    {
                        my $cfg_line_str = join(":",@cfg_line);
                        print "\nInvalid activity-package definitions '$cfg_line_str'\n";
                        print "Unique SWD package '$unique_package' set togheter with single acitivities\n\n";
                        exit 6;
                    }
                    else
                    {
                        if (!($act_hash{$t_act}))
                        {
                            $act_hash{$t_act} = $t_pack;
                        }
                        else
                        {
                            my $cfg_line_str = join(":",@cfg_line);
                            print "\nInvalid activity-package definitions '$cfg_line_str'\n";
                            print "Two SWD packages ('$act_hash{$t_act}' and '$t_pack') set for Activity '$t_act'\n\n";
                            exit 7;
                        }     
                    }
                }
            }
        }

        my ($numacts, $totalacts);
        my $targfile = $working_dir . "/Targets/${file}.txt";
        my $numtargs = get_num_targs($targfile, $exclusion_flag, $exclusion_list);

        open(XML,"< ${working_dir}/$xml") 
          or die "can't open  ${working_dir}/$xml: $!\n";
        chomp(my @xml = <XML>);
        close(XML);

        $numacts = grep(/<\/activity>/,@xml);

        $totalacts = $numtargs * $numacts;

        if (!($xml_list{"${pre}_${id}_${plan_xml_name}"}))
        {
            $xml_list{ "${pre}_${id}_${plan_xml_name}" } = 1;
        }
        else
        {
            print "\n${pre}_${id}_${plan_xml_name} already exists\n";
            print "You need to validate your *.cfg files\n\n";
            exit 8;
        }     

        open(PLAN, "> ${working_dir}/${pre}/${pre}_${id}_${plan_xml_name}.xml") 
          or die "can't open ${pre}_${id}_${plan_xml_name}.xml : $!\n";

        print "\nCreating XML file: ${pre}_${id}_${plan_xml_name}.xml...\n";

        $grand_parent_line = shift(@xml);
        $parent_line = shift(@xml);
        $current_line = shift(@xml);

        my $line_count = 0;

        print PLAN $grand_parent_line."\n".$parent_line."\n";

        undef my $activity_name;
        foreach $line (@xml)
        {
            if ($activity_name && $current_line =~ m/<\/activity>/)
            {
                undef $activity_name;
            }
            elsif (($grand_parent_line =~ m/<activity>/) && 
                ($current_line =~ m/<name>(.+)<\/name>/))
            {
                $activity_name = $1;
            }
            elsif (($parent_line =~ m/<activity_plan>/) && 
                ($current_line =~ m/<name>(.+)<\/name>/))
            {
                $current_line =~ s/$1/${pre}_${id}_${plan_xml_name}/;
                $cng_1_done = 1;
                print "Set Plan name = '${pre}_${id}_${plan_xml_name}'\n";
            }
            elsif(($parent_line =~ m/<target_resource_type>endpoint<\/target_resource_type>/) && ($current_line =~ m/<targets type=\"file\">(.+)<\/targets>/))
            {
                $current_line =~ s/$1/\@$targ_mn:$targfile/;
                print "Set Target file = '\@$targ_mn:$targfile'\n";
                $cng_2_done = 1;
            }
            elsif(($parent_line =~ m/<name>FromDepot<\/name>/) && 
                  ($current_line =~ m/<value>(.+)<\/value>/))
            {
                $current_line =~ s/$1/$depot/;
                $cng_3_done = 1;
                print "Set From Depot = '$depot'\n";
            }
            elsif(($parent_line =~ m/<name>SoftwarePackage<\/name>/) &&
                  ($current_line =~ m/<value>(.+)<\/value>/))
            {
                my $curr_pack = $unique_package;
                if (!$curr_pack || $curr_pack eq "")
                {
                    $curr_pack = $act_hash{$activity_name};
                }
                if ($curr_pack && $curr_pack ne "")
                {
                    $current_line =~ s/$1/$curr_pack/;
                    print "Set SWD Package = '$curr_pack' for activity $activity_name\n";
                }
                else
                {
                    $curr_pack = $1;
                    print "Left SWD Package = '$curr_pack' for activity $activity_name\n";
                }
                if ($swdpack_check == 1)
                {
                    `wlookup -r SoftwarePackage $curr_pack > $nullDevice`;
                    if ( $? != 0 )
                    {
                        print "\nSoftwarePackage $curr_pack Not Found\n\n";
                        exit 9;
                    }
                }
            }

            print PLAN $current_line . "\n";

            $grand_parent_line = $parent_line;
            $parent_line = $current_line;
            $current_line = $line;
            $line_count++;
        }

        print PLAN $current_line . "\n";
        close(PLAN);

        if (($cng_1_done == 1) && ($cng_2_done == 1) && ($cng_3_done == 1))
        {
            print "Created ${pre}_${id}_${plan_xml_name}.xml\n\n";
        }
        else
        {
            print "\nAll parameters were not updated \n\n";
            exit 10;
        }

 
        if ( $response =~ /y/i )
        {
            `wimppln -o -f ${working_dir}/${pre}/${pre}_${id}_${plan_xml_name}.xml`;
        }

        open(PLNLST, ">> ${working_dir}/${pre}/${pre}_dist_pln.lst") 
          or die "can't open ${pre}_dist_pln.lst: $!\n";
        print PLNLST "${pre}_${id}_${plan_xml_name},Not_Submitted,NoId,$totalacts,$numtargs\n";
        close(PLNLST);
    }
}

#
# Exit section
#
exit 0;

################################################################
# Sub Routines ################################################################

sub get_config
{
    my $file = shift;
    my @cfg_arry;

    open(IN, "< $file") || die "can't open $file: $!\n";
    while (<IN>)
    {
        chomp;
        my @line_arry = split(":", $_);
        push(@cfg_arry, \@line_arry); 
    }
    close(IN);

    return(@cfg_arry);
}

sub get_num_targs
{
    my $targfile = shift;
    my $exclusion_flag = shift;
    my $exclusion_list = shift;

    my $numtargs = 0;
    my @temptarg;

    open(TARG, "< $targfile") or die "can't open $targfile: $!\n";
    chomp(my @targets = <TARG>);
    close(TARG);

    if ( $exclusion_flag =~ /y/i ) 
    {
        open(EXC, "< $exclusion_list") or die "cant open $exclusion_list: $!\n";
        chomp(my @exclusion = <EXC>);
        close(EXC);

        foreach my $target (@targets) 
        {
            if ( grep(!/$target/,@exclusion) )
            {
                push(@temptarg,$target);
            }
        }
        $numtargs = scalar(@temptarg);
    } 
    else 
    {
        $numtargs = scalar(@targets);
    }

    return($numtargs);

}

sub get_init
{
    my $file = shift;

    if ( -e $file )
    {
        open(IN, "< $file");
        while (<IN>)
        {
            if (!($_ =~ /^ *#/) && !($_ =~ /^ *$/ ))
            {
            chomp;
                if (index($_, "=") != -1)
                {
                    my @init_line = split("=", $_);
                    $init_line[0] =~ s/ |\t//g;
                    if ( $init_line[0] && $init_line[0] eq "ActivityDelimiter" )
                    {
                        if (!$init_line[1])
                        {
                            print "\nThe ActivityDelimiter character cannot be NULL\n\n";
                            exit 11;
                        }
                        elsif ($init_line[1] =~/[\$@\t\\_:#a-zA-Z0-9]/ || $init_line[1] eq "\"" || $init_line[1] eq "'")
                        {
                            print "\nThe '$init_line[1]' character cannot be used as ActivityDelimiter\n\n";
                            exit 11;
                        }
                        if ($init_line[1] =~/"/ || $init_line[1] =~/'/ )
                        {
                            $act_delimiter = `echo $init_line[1] 2>$nullDevice`;
                            chomp($act_delimiter);
                        }
                        else
                        {
                            $act_delimiter = $init_line[1];
                        }
                        if (!$act_delimiter || $act_delimiter eq "" || length($act_delimiter) != 1)
                        {
                            print "\nThe '$init_line[1]' character cannot be used as ActivityDelimiter\n\n";
                            exit 11;
                        }
                    }
                    elsif ($init_line[0])
                    {
                        my $this_env = "";
                        if ( $init_line[1] && $init_line[1] ne "" )
                        {
                            $init_line[1] =~ s/\t*$| *$//;
                            if ( $init_line[1] && $init_line[1] ne "" )
                            {
                                $this_env = `echo "$init_line[1]"`;
                                chomp($this_env);
                            }
                        }
                        $ENV{$init_line[0]} = $this_env;
                        if ( $init_line[0] eq "WorkDir" )
                        {
                            $working_dir = $ENV{"WorkDir"};
                            $working_dir =~ s/\/$//;
                        }
                        if ( $init_line[0] eq "ExclusionList" )
                        {
                            $exclusion_list = $this_env;
                        }
                        if ( $init_line[0] eq "SWDPackageCheck" )
                        {
                            if ( !$this_env || $this_env =~ /^n/i || $this_env =~/^f/i || $this_env eq "0" )
                            {
                                $swdpack_check = 0;
                            }
                        }
                    }
                }
            }
        }
        close(IN);
    }
}

sub check_and_create_query_library
{
    my $query_lib_name_obj = `wlookup -r QueryLibrary $query_lib_name 2>$nullDevice`;
    if ( $? != 0 )
    {
        my(@policy_list) = `wlookup -ar PolicyRegion`;
        my $np = @policy_list;
        my ($risp) = -2; my $rr; my $pol_name; my $pol_obj;

        if ( $np == 1 )
        {
            $risp = 0;
            ($pol_name, $pol_obj) = split(" ", $policy_list[$risp]);
            print "Do you want to create the Query Library $query_lib_name\nin the Policy Region $pol_name? (y/n)\n";
	    $rr = <STDIN>;
            if ( !($rr =~ /^y/i) )
            {
                exit 0;
            }
        }
        elsif ( $np > 1 )
        {
            my($ind) = 0;
            $rr = 0;

            while ( $risp == -2 )
            {
                print "\nThe Query Library $query_lib_name is missing;\n\n";
                $ind = 0;
                foreach (@policy_list)
                {
                    $ind++;
                    ($pol_name, $pol_obj) = split(" ", $_);
                    print "\t$ind\t- $pol_name\n";
                }

                print "\nPlease choose a Policy Region [1-$np] where the Query Library will be created.\n";
                print "Choose 0 to exit:\n";
                $rr = <STDIN>;
                chomp $rr;
                if ( $rr == 0 )
                {
                    exit 0;
                }

                $risp = $rr - 1;

                if ( $risp > $np )
                {
                    $risp = -2;
                }
            }
        }
        else 
        {
            print "\nThe Query Library $query_lib_name is missing,\n";
            print "but no Policy Region can be found in this TMR;\n";
            print "please, solve the problem (create a Policy Region) ";
            print "and run again this script.\n\n";
            exit 12;
        }

        ($pol_name, $pol_obj) = split(" ", $policy_list[$risp]);
        print "Creating the Query Library $query_lib_name in Policy Region $pol_name\n";

        `wcrtqlib $pol_name $query_lib_name`;
        if ( $? !=  0 )
        {
            print "\nUnable to create the Query Library $query_lib_name. Exiting\n\n";
            exit 13;
        }
    }
}

################################################################;
# Error Routines ################################################################;
#NodeNameQ=`idlcall $LocalHostOid SysAdmin::PolicyDrivenBase::_get_label`
#LocalHostOid=`objcall 0.0.0 getattr HostLocation`
